// $SBND: appdev/SRI/ParamShopCart/ConnectionUtil/ConnectionProperties.cs,v 1.4 2003/06/24 10:40:38 roam Exp $

using System;

using Ringlet.Misc.VCS;

namespace Ringlet.Data.ConnectionUtil
{
	/// <summary>
	/// A data structure containing the data needed to
	/// establish a connection to a SQL database.
	/// </summary>
	public class ConnectionProperties
	{
		/// <summary>
		/// The SQL server hostname
		/// </summary>
		protected string server;
		/// <summary>
		/// The database name on the SQL server
		/// </summary>
		protected string database;
		/// <summary>
		/// The username for the SQL connection
		/// </summary>
		protected string username;
		/// <summary>
		/// The password for the SQL connection
		/// </summary>
		protected string password;

		/// <summary>
		/// Default constructor: initialize with values
		/// suitable for a default ParamShopCart install
		/// </summary>
		public ConnectionProperties()
		{
			server = "localhost";
			database = "Store";
			username = password = String.Empty;
		}

		/// <summary>
		/// The version control system tag.
		/// </summary>
		public virtual string VCSTag
		{
			get
			{
				return ("$Ringlet: cs/misc/Scramble/ConnectionUtil/ConnectionProperties.cs,v 1.2 2003/07/08 12:22:53 roam Exp $");
			}
		}

		/// <summary>
		/// Get or set the SQL server name
		/// </summary>
		public string Server
		{
			get
			{
				return (server);
			}

			set
			{
				server = value;
			}
		}

		/// <summary>
		/// Get or set the SQL database name
		/// </summary>
		public string Database
		{
			get
			{
				return (database);
			}

			set
			{
				database = value;
			}
		}

		/// <summary>
		/// Get or set the SQL username
		/// </summary>
		public string Username
		{
			get
			{
				return (username);
			}

			set
			{
				username = value;
			}
		}

		/// <summary>
		/// Get or set the SQL password
		/// </summary>
		public string Password
		{
			get
			{
				return (password);
			}

			set
			{
				password = value;
			}
		}

		/// <summary>
		/// Copy a set of connection properties to another
		/// ConnectionProperties object.
		/// </summary>
		/// <param name="dst">the object to copy to</param>
		public void Copy(ConnectionProperties dst)
		{
			dst.server = server;
			dst.database = database;
			dst.username = username;
			dst.password = password;
		}
	}
}
