using System;
using System.Collections;
using System.Configuration;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Web;
using System.Web.SessionState;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.HtmlControls;

using Ringlet.Data.ConnectionUtil;
using Ringlet.Drawing.Scramble;
using Ringlet.Misc.VCS;

namespace ScrambleWebApp
{
	/// <summary>
	/// The Scramble start page: choose a puzzle, view high scores.
	/// </summary>
	public class Default : System.Web.UI.Page, IVersioned
	{
		/// <summary>
		/// The placeholder for the puzzles table.
		/// </summary>
		protected System.Web.UI.WebControls.PlaceHolder plcTable;
		/// <summary>
		/// The request analysis utility.
		/// </summary>
		protected ScWebUtil scutil;
		/// <summary>
		/// The SQL connection properties.
		/// </summary>
		protected ConnectionProperties prop;
		/// <summary>
		/// The puzzle database access object.
		/// </summary>
		protected SCDatabase db;
		/// <summary>
		/// The images loaded from the puzzle database.
		/// </summary>
		protected SCImageData[] imgdata;

		/// <summary>
		/// Get the version control system tag.
		/// </summary>
		public virtual string VCSTag {
			get
			{
				return ("$Ringlet: cs/misc/Scramble/Scramble/Default.aspx.cs,v 1.3 2003/07/11 18:13:23 roam Exp $");
			}
		}
	
		/// <summary>
		/// Initialize the ASP.NET page.
		/// </summary>
		/// <param name="sender">The sender control (unused)</param>
		/// <param name="e">Event arguments (unused)</param>
		private void Page_Load(object sender, System.EventArgs e)
		{
		
			scutil = new ScWebUtil(Request);
			prop = scutil.ConnectionProperties;

			Session.Remove("pic_id");
			Session.Remove("newgame");
			if (Request.Params["pic_id"] != null)
				ProcessRequest();
			DisplayPics();
		}

		/// <summary>
		/// Display the available puzzle images.
		/// </summary>
		protected void DisplayPics()
		{
			HtmlTable tbl;
			HtmlTableRow row;
			HtmlTableCell cell;
			HtmlAnchor a;
			HtmlImage himg;
			//Control c;
			Literal l;
			int pos;

			try
			{
				db = new SCDatabase();
				db.ConnectionProperties = prop;
				imgdata = db.ListAllImages();
			}
			catch (Exception e)
			{
				Response.Write("<p>FIXME: an error occurred: " +
				    HttpUtility.HtmlEncode(e.Message) + "</p>\n");
				return;
			}

			tbl = new HtmlTable();
			pos = 0;
			row = null;
			foreach (SCImageData img in imgdata)
			{
				if (pos == 0)
					row = new HtmlTableRow();
				cell = new HtmlTableCell();
				himg = new HtmlImage();
				himg.Src = "ShowThumbnail.aspx?pic_id=" + img.id.ToString();
				himg.Alt = "Picture: " + img.name;
				a = new HtmlAnchor();
				a.Attributes.Add("href", "?pic_id=" + img.id.ToString());
				a.Controls.Add(himg);
				cell.Controls.Add(a);
				/*
				c = Page.ParseControl("<p>" + HttpUtility.HtmlEncode(img.name) +
				    ": " + img.cols.ToString() + "/" + img.rows.ToString() + "</p>");
				cell.Controls.Add(c);
				*/
				l = new Literal();
				l.Text = String.Format("<p>{0} ({1}x{2})</p>",
				    img.name, img.cols, img.rows);
				cell.Controls.Add(l);
				row.Cells.Add(cell);
				if (pos == 2)
				{
					tbl.Rows.Add(row);
					row = null;
				}
				pos = (pos + 1) % 3;
			}
			if (row != null)
				tbl.Rows.Add(row);
			plcTable.Controls.Add(tbl);
		}

		/// <summary>
		/// Process the user's puzzle request.
		/// </summary>
		protected void ProcessRequest()
		{
			int id;
			SCDatabase db;
			SCImageData img;

			try
			{
				id = int.Parse(Request.Params["pic_id"]);
				db = new SCDatabase();
				db.ConnectionProperties = prop;
				img = db.ListImage(id);
				if (img.id == -1)
					throw (new ApplicationException(
					    "Unknown image id: " +
					    id.ToString()));
				Session["pic_id"] = id;
				Session["newgame"] = true;
				Response.Redirect("Puzzle.aspx");
			}
			catch (Exception e)
			{
				Response.Write("<p>FIXME: Error: " + e.Message);
			}
		}

		#region Web Form Designer generated code
		/// <summary>
		/// System initialization of the ASP.NET page object.
		/// </summary>
		/// <param name="e">Event arguments</param>
		override protected void OnInit(EventArgs e)
		{
			//
			// CODEGEN: This call is required by the ASP.NET Web Form Designer.
			//
			InitializeComponent();
			base.OnInit(e);
		}
		
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{    
			this.Load += new System.EventHandler(this.Page_Load);

		}
		#endregion
	}
}
