using System;
using System.Collections;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Web;
using System.Web.SessionState;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.HtmlControls;

using Ringlet.Data.ConnectionUtil;
using Ringlet.Drawing.Scramble;
using Ringlet.Misc.VCS;

namespace ScrambleWebApp
{
	/// <summary>
	/// Display the high scores table.
	/// </summary>
	public class HighScores : System.Web.UI.Page, IVersioned
	{
		/// <summary>
		/// The request analysis utility.
		/// </summary>
		protected ScWebUtil scutil;
		/// <summary>
		/// The SQL connection properties.
		/// </summary>
		protected ConnectionProperties prop;
		/// <summary>
		/// The number of high scores to display.
		/// </summary>
		protected int top;
		/// <summary>
		/// The currently selected scores table image rows.
		/// </summary>
		protected int rows;
		/// <summary>
		/// The currently selected scores table image columns.
		/// </summary>
		protected int cols;
		/// <summary>
		/// The error message ASP.NET label.
		/// </summary>
		protected System.Web.UI.WebControls.Label lblError;
		/// <summary>
		/// The placeholder for the high scores table.
		/// </summary>
		protected System.Web.UI.WebControls.PlaceHolder plcTable;
		/// <summary>
		/// The congratulations label.
		/// </summary>
		protected System.Web.UI.WebControls.Label lblCongrats;
		/// <summary>
		/// The 'enter your name' label.
		/// </summary>
		protected System.Web.UI.WebControls.Label lblName;
		/// <summary>
		/// The player name entry textbox.
		/// </summary>
		protected System.Web.UI.WebControls.TextBox txtName;
		/// <summary>
		/// The 'add score' button.
		/// </summary>
		protected System.Web.UI.WebControls.Button btnAdd;
		/// <summary>
		/// The 'Rows' label.
		/// </summary>
		protected System.Web.UI.WebControls.Label lblRows;
		/// <summary>
		/// The image rows textbox.
		/// </summary>
		protected System.Web.UI.WebControls.TextBox txtRows;
		/// <summary>
		/// The 'Columns' label.
		/// </summary>
		protected System.Web.UI.WebControls.Label lblCols;
		/// <summary>
		/// The image columns textbox.
		/// </summary>
		protected System.Web.UI.WebControls.TextBox txtColumns;
		/// <summary>
		/// The 'display scores' button.
		/// </summary>
		protected System.Web.UI.WebControls.Button btnDisplay;
		/// <summary>
		/// The 'Display Top' label.
		/// </summary>
		protected System.Web.UI.WebControls.Label lblTop;
		/// <summary>
		/// The scores number textbox.
		/// </summary>
		protected System.Web.UI.WebControls.TextBox txtTop;
		/// <summary>
		/// The high scores database access object.
		/// </summary>
		protected SCHighScores sc;
		
		/// <summary>
		/// Initialize the ASP.NET page.
		/// </summary>
		/// <param name="sender">Sender object (may be btnDisplay)</param>
		/// <param name="e">Event arguments (unused)</param>
		private void Page_Load(object sender, System.EventArgs e)
		{

			scutil = new ScWebUtil(Request);
			prop = scutil.ConnectionProperties;
		
			if (Request.Form["btnDisplay"] != null &&
			    sender != btnDisplay)
			{
				// Let btnDisplay do its own dirty work...
				ShowCongrats(false);
				return;
			}
			top = rows = cols = -1;
			GatherScoreSize();
			if (top == -1 || rows == -1 || cols == -1)
			{
				ShowCongrats(false);
				return;
			}
			txtRows.Text = rows.ToString();
			txtColumns.Text = cols.ToString();
			txtTop.Text = top.ToString();
			
			GatherScores();

			// Only display the 'Congratulations' box when
			// the moves are set *and* the name is not yet set.
			if (Session["sc_moves"] == null ||
			    Session["sc_name"] != null)
				ShowCongrats(false);
			DisplayScores();
		}

		/// <summary>
		/// The version control system tag.
		/// </summary>
		public virtual string VCSTag
		{
			get
			{
				return ("$Ringlet: cs/misc/Scramble/Scramble/HighScores.aspx.cs,v 1.1 2003/07/11 18:36:31 roam Exp $");
			}
		}

		/// <summary>
		/// Obtain the number of rows, columns, and scores to display
		/// from the user's ASP.NET session.
		/// </summary>
		public void GatherScoreSize()
		{
			try
			{
				top = (int)Session["sc_top"];
				rows = (int)Session["sc_rows"];
				cols = (int)Session["sc_cols"];
			}
			catch {}
		}

		/// <summary>
		/// Retrieve the selected high scores table.
		/// </summary>
		public void GatherScores()
		{
			try
			{
				sc = new SCHighScores();
				sc.ConnectionProperties = prop;
				sc.Rows = rows;
				sc.Columns = cols;
				sc.ScoreNum = top;
				sc.SelectTop();
			}
			catch (Exception e)
			{
				lblError.Text += "Error: " + e.Message + " <br />\n";
			}
		}

		/// <summary>
		/// Display the selected high scores table.
		/// </summary>
		public void DisplayScores()
		{
			HtmlTable tbl;
			HtmlTableRow row;
			HtmlTableCell cell;
			SCScore[] scores;
			Literal l;
			int i;
			
			scores = sc.Scores;
			tbl = new HtmlTable();
			tbl.ID = "tblScores";

			// Add the heading row (ugly)
			row = new HtmlTableRow();
			cell = new HtmlTableCell();
			l = new Literal();
			l.Text = "Position";
			cell.Controls.Add(l);
			row.Cells.Add(cell);
			cell = new HtmlTableCell();
			l = new Literal();
			l.Text = "Name";
			cell.Controls.Add(l);
			row.Cells.Add(cell);
			cell = new HtmlTableCell();
			l = new Literal();
			l.Text = "Moves";
			cell.Controls.Add(l);
			row.Cells.Add(cell);
			cell = new HtmlTableCell();
			l = new Literal();
			l.Text = "Date";
			cell.Controls.Add(l);
			row.Cells.Add(cell);
			tbl.Rows.Add(row);

			for (i = 0; i < scores.Length; i++)
			{
				row = new HtmlTableRow();

				cell = new HtmlTableCell();
				l = new Literal();
				l.Text = (i + 1).ToString();
				cell.Controls.Add(l);
				row.Cells.Add(cell);

				cell = new HtmlTableCell();
				l = new Literal();
				l.Text = scores[i].name;
				cell.Controls.Add(l);
				row.Cells.Add(cell);

				cell = new HtmlTableCell();
				l = new Literal();
				l.Text = scores[i].moves.ToString();
				cell.Controls.Add(l);
				row.Cells.Add(cell);

				cell = new HtmlTableCell();
				l = new Literal();
				l.Text = scores[i].date.ToString();
				cell.Controls.Add(l);
				row.Cells.Add(cell);

				// ..and add it to the table.
				tbl.Rows.Add(row);
			}
			foreach (Control c in plcTable.Controls)
			{
				if (c is HtmlTable && c.ID == tbl.ID)
				{
					plcTable.Controls.Remove(c);
					break;
				}
			}
			plcTable.Controls.Add(tbl);
		}

		#region Web Form Designer generated code
		/// <summary>
		/// System initialization of the ASP.NET page object.
		/// </summary>
		/// <param name="e">Event arguments</param>
		override protected void OnInit(EventArgs e)
		{
			//
			// CODEGEN: This call is required by the ASP.NET Web Form Designer.
			//
			InitializeComponent();
			base.OnInit(e);
		}
		
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{    
			this.btnAdd.Click += new System.EventHandler(this.btnAdd_Click);
			this.btnDisplay.Click += new System.EventHandler(this.btnDisplay_Click);
			this.Load += new System.EventHandler(this.Page_Load);

		}
		#endregion

		/// <summary>
		/// Handle the 'Add' button: update the high scores table.
		/// </summary>
		/// <param name="sender">Sender control (unused)</param>
		/// <param name="e">Event arguments (unused)</param>
		private void btnAdd_Click(object sender, System.EventArgs e)
		{
			int moves;
			SCScore s;

			if (txtName.Text == String.Empty)
			{
				lblError.Text += "No name specified for adding to the high score list! <br />\n";
				return;
			}

			try
			{
				
				moves = (int)Session["sc_moves"];
				if (!sc.IsHighScore(moves))
					return;
				s = new SCScore();
				s.rows = (int)Session["sc_rows"];
				s.cols = (int)Session["sc_cols"];
				s.moves = moves;
				s.name = txtName.Text;
				s.picture = (string)Session["sc_picture"];
				s.date = DateTime.Now;
				sc.UpdateScore(s);
				GatherScores();
				DisplayScores();

				ShowCongrats(false);
			}
			catch (Exception ex)
			{
				lblError.Text += "Error: " + ex.Message + " <br />\n";
			}
		}

		/// <summary>
		/// Handle the 'Display' button: display another high scores table
		/// (other values for rows, columns, and scores number).
		/// </summary>
		/// <param name="sender">Sender control (may be passed to Page_Load())</param>
		/// <param name="e">Event arguments (unused)</param>
		private void btnDisplay_Click(object sender, System.EventArgs e)
		{
			try
			{
				Session["sc_rows"] = int.Parse(txtRows.Text);
				Session["sc_cols"] = int.Parse(txtColumns.Text);
				Session["sc_top"] = int.Parse(txtTop.Text);
				// Just in case...
				ShowCongrats(true);
				// Yuck :)
				Page_Load(sender, null);
			}
			catch (Exception ex)
			{
				lblError.Text += "Error: " + ex.Message + " <br />\n";
			}
		}

		/// <summary>
		/// Show or hide the congratulations.
		/// </summary>
		/// <param name="val">Boolean flag: show or hide.</param>
		protected void ShowCongrats(bool val)
		{
			lblCongrats.Visible = val;
			lblName.Visible = val;
			txtName.Visible = val;
			btnAdd.Visible = val;
		}
	}
}
