using System;
using System.Collections;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Drawing.Imaging;
using System.IO;
using System.Web;
using System.Web.SessionState;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.HtmlControls;

using Ringlet.Data.ConnectionUtil;
using Ringlet.Drawing.Scramble;
using Ringlet.Misc.VCS;

namespace ScrambleWebApp
{
	/// <summary>
	/// Display the image of a puzzle fragment.
	/// </summary>
	public class PuzzleFrag : System.Web.UI.Page, IVersioned
	{

		/// <summary>
		/// Initialize the ASP.NET page and display the image.
		/// </summary>
		/// <param name="sender">Sender control (unused)</param>
		/// <param name="e">Event arguments (unused)</param>
		private void Page_Load(object sender, System.EventArgs e)
		{
			ScWebUtil scutil;
			ConnectionProperties prop;
			SCPuzzle pz;
			System.Drawing.Image img;
			MemoryStream str;
			int pzid;
			uint x, y;

			str = null;
			try
			{
				scutil = new ScWebUtil(Request);
				prop = scutil.ConnectionProperties;

				pzid = int.Parse(Request.Params["pzid"]);
				x = uint.Parse(Request.Params["x"]);
				y = uint.Parse(Request.Params["y"]);

				pz = new SCPuzzle();
				pz.ConnectionProperties = prop;
				img = pz.LoadFragmentImage(pzid, x, y);
				if (Request.Params["blank"] != null)
				{
					System.Drawing.Image nimg;

					nimg = new Bitmap(img.Width, img.Height);
					img = nimg;
				}
				str = new MemoryStream();
				img.Save(str, ImageFormat.Jpeg);
				Response.BinaryWrite(str.GetBuffer());
				str.Close();
				str = null;
			}
			catch (Exception ex)
			{
				Response.Write("<html><body><p>\nError: " +
					HttpUtility.HtmlEncode(ex.Message) +
					"\n</p></body></html>\n");
			}
			finally
			{
				if (str != null)
					try
					{
						str.Close();
					}
					catch {}
			}
		}

		/// <summary>
		/// Get the version control system tag.
		/// </summary>
		public virtual string VCSTag
		{
			get
			{
				return ("$Ringlet: cs/misc/Scramble/Scramble/PuzzleFrag.aspx.cs,v 1.2 2003/07/11 18:13:23 roam Exp $");
			}
		}

		#region Web Form Designer generated code
		/// <summary>
		/// System initialization of the ASP.NET page object.
		/// </summary>
		/// <param name="e"></param>
		override protected void OnInit(EventArgs e)
		{
			//
			// CODEGEN: This call is required by the ASP.NET Web Form Designer.
			//
			InitializeComponent();
			base.OnInit(e);
		}
		
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{    
			this.Load += new System.EventHandler(this.Page_Load);
		}
		#endregion
	}
}
