using System;
using System.Configuration;
using System.Web;

using Ringlet.Data.ConnectionUtil;
using Ringlet.Misc.VCS;

namespace ScrambleWebApp
{
	/// <summary>
	/// Analyze the HTTP request and gather configuration data.
	/// </summary>
	public class ScWebUtil : IVersioned
	{
		/// <summary>
		/// The HTTP request to analyze.
		/// </summary>
		protected HttpRequest req;

		/// <summary>
		/// Initialize a ScWebUtil object with default values.
		/// </summary>
		public ScWebUtil()
		{
			req = null;
		}

		/// <summary>
		/// Initialize a ScWebUtil object with an HTTP request object.
		/// </summary>
		/// <param name="nreq">The request to analyze.</param>
		public ScWebUtil(HttpRequest nreq) : this()
		{
			req = nreq;
		}

		/// <summary>
		/// The version control system tag.
		/// </summary>
		public virtual string VCSTag
		{
			get
			{
				return ("$Ringlet: cs/misc/Scramble/Scramble/ScWebUtil.cs,v 1.2 2003/07/11 18:13:23 roam Exp $");
			}
		}

		/// <summary>
		/// Get the connection properties derived from the request and
		/// the Web.config file.
		/// </summary>
		public ConnectionProperties ConnectionProperties
		{
			get
			{
				ConnectionProperties prop;

				prop = new ConnectionProperties();
				prop.Server = ConfigurationSettings.AppSettings["conn_server"];
				prop.Database = ConfigurationSettings.AppSettings["conn_database"];
				prop.Username = ConfigurationSettings.AppSettings["conn_username"];
				prop.Password = ConfigurationSettings.AppSettings["conn_password"];
				return (prop);
			}
		}
	}
}
