using System;
using System.Collections;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Drawing.Imaging;
using System.IO;
using System.Web;
using System.Web.SessionState;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.HtmlControls;

using Ringlet.Data.ConnectionUtil;
using Ringlet.Drawing.Scramble;
using Ringlet.Misc.VCS;

namespace ScrambleWebApp
{
	/// <summary>
	/// Display a puzzle image's thumbnail.
	/// </summary>
	public class ShowThumbnail : System.Web.UI.Page, IVersioned
	{
		/// <summary>
		/// The request analyzer.
		/// </summary>
		protected ScWebUtil scutil;
		/// <summary>
		/// The SQL connection properties.
		/// </summary>
		protected ConnectionProperties prop;

		/// <summary>
		/// Initialize the ASP.NET page and display the image.
		/// </summary>
		/// <param name="sender">Sender control (unused)</param>
		/// <param name="e">Event arguments (unused)</param>
		private void Page_Load(object sender, System.EventArgs e)
		{
			int id;
			SCDatabase db;
			SCImageData imgdata;
			System.Drawing.Image img, thumb;
			FileStream fs;
			MemoryStream m;
	
			fs = null;
			m = null;
			try
			{
				scutil = new ScWebUtil(Request);
				prop = scutil.ConnectionProperties;
				if (Request.Params["pic_id"] == null)
					throw (new ApplicationException("No picture ID provided"));
				id = int.Parse(Request.Params["pic_id"]);
				db = new SCDatabase();
				db.ConnectionProperties = prop;
				imgdata = db.ListImage(id);
				if (imgdata.id == -1)
					throw (new ApplicationException("No image found with id " + Request.Form["pic_id"]));
				fs = new FileStream(imgdata.filename, FileMode.Open);
				img = new Bitmap(fs);
				fs.Close();
				fs = null;
				thumb = new Bitmap(img, new Size(100, 100));
				m = new MemoryStream();
				thumb.Save(m, ImageFormat.Jpeg);
				Response.BinaryWrite(m.GetBuffer());
			}
			catch (Exception ex)
			{
				Response.Write("<html><body><p>\n" +
					HttpUtility.HtmlEncode(ex.Message) +
					"\n</p></body></html>\n");
			}
			finally
			{
				if (m != null)
					try
					{
						m.Close();
					}
					catch {}
				if (fs != null)
					try
					{
						fs.Close();
					}
					catch {}
			}
		}

		/// <summary>
		/// Get the version control system tag.
		/// </summary>
		public virtual string VCSTag
		{
			get
			{
				return ("$Ringlet: cs/misc/Scramble/Scramble/ShowThumbnail.aspx.cs,v 1.2 2003/07/11 18:13:23 roam Exp $");
			}
		}

		#region Web Form Designer generated code
		/// <summary>
		/// System initialization of the ASP.NET object.
		/// </summary>
		/// <param name="e">Event arguments</param>
		override protected void OnInit(EventArgs e)
		{
			//
			// CODEGEN: This call is required by the ASP.NET Web Form Designer.
			//
			InitializeComponent();
			base.OnInit(e);
		}
		
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{    
			this.Load += new System.EventHandler(this.Page_Load);
		}
		#endregion
	}
}
