using System;
using System.Data;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;

using Ringlet.Data.ConnectionUtil;
using Ringlet.Drawing.Scramble;
using Ringlet.Misc.VCS;

namespace Ringlet.Misc.Scramble.Unscramble
{
	/// <summary>
	/// Display the high score table.
	/// </summary>
	public class HighScores : System.Windows.Forms.Form, IVersioned
	{
		/// <summary>
		/// Required designer variable.
		/// </summary>
		private System.ComponentModel.Container components = null;
		/// <summary>
		/// The Retrieve button.
		/// </summary>
		private System.Windows.Forms.Button btnRetrieve;
		private System.Windows.Forms.Label lblRows;
		private System.Windows.Forms.Label lblCols;
		private System.Windows.Forms.Label lblTop;
		private System.Windows.Forms.TextBox txtRows;
		private System.Windows.Forms.TextBox txtCols;
		private System.Windows.Forms.TextBox txtTop;
		private System.Windows.Forms.DataGrid dgScores;
		private System.Windows.Forms.Button btnOK;
		/// <summary>
		/// The web service to use.
		/// </summary>
		protected PuzzleData svc;
		/// <summary>
		/// The puzzle object to obtain data from.
		/// </summary>
		protected SCPuzzle puzzle;
		/// <summary>
		/// The high score database object.
		/// </summary>
		protected SCHighScores high;
		/// <summary>
		/// The number of image rows for the current score table.
		/// </summary>
		protected int rows;
		/// <summary>
		/// The number of image columns for the current score table.
		/// </summary>
		protected int cols;
		/// <summary>
		/// The number of high score records in the current table.
		/// </summary>
		protected int top;
		/// <summary>
		/// Flag: do we need to check for a high score right now?
		/// </summary>
		protected bool scored;

		/// <summary>
		/// Initialize a HighScores form with default values.
		/// </summary>
		public HighScores()
		{
			//
			// Required for Windows Form Designer support
			//
			InitializeComponent();

			high = new SCHighScores();
			high.AddScoreDlg = new SCHighScores.AddScoreDelegate(SvcAddScore);
			high.DeleteScoreDlg = new SCHighScores.DeleteScoreDelegate(SvcDeleteScore);
			high.ListAllScoresDlg = new SCHighScores.ListAllScoresDelegate(SvcListScores);
			rows = cols = -1;
			top = 10;
			txtRows.Text = txtCols.Text = String.Empty;
			txtTop.Text = "10";
			scored = false;
		}

		/// <summary>
		/// Get the version control system tag.
		/// </summary>
		public virtual string VCSTag
		{
			get
			{
				return ("$Ringlet: cs/misc/Scramble/Unscramble/HighScores.cs,v 1.1 2003/07/12 01:55:39 roam Exp $");
			}
		}

		/// <summary>
		/// Get or set the web service object.
		/// </summary>
		public PuzzleData Service
		{
			get
			{
				return (svc);
			}

			set
			{
				svc = value;
			}
		}

		/// <summary>
		/// Get or set the puzzle.
		/// </summary>
		public SCPuzzle Puzzle
		{
			get
			{
				return (puzzle);
			}

			set
			{
				puzzle = value;
				Rows = (int)puzzle.Image.Image.Rows;
				Columns = (int)puzzle.Image.Image.Columns;
				btnRetrieve_Click(this, null);
			}
		}

		/// <summary>
		/// Get or set the number of image rows.
		/// </summary>
		public int Rows
		{
			get
			{
				return (rows);
			}

			set
			{
				rows = value;
				txtRows.Text = rows.ToString();
			}
		}

		/// <summary>
		/// Get or set the number of image columns.
		/// </summary>
		public int Columns
		{
			get
			{
				return (cols);
			}

			set
			{
				cols = value;
				txtCols.Text = cols.ToString();
			}
		}

		/// <summary>
		/// Get or set the number of high score entries displayed.
		/// </summary>
		public int ScoreNum
		{
			get
			{
				return (top);
			}

			set
			{
				top = value;
				txtTop.Text = top.ToString();
			}
		}

		/// <summary>
		/// Get or set the enter-new-highscore flag.
		/// </summary>
		public bool Scored
		{
			get
			{
				return (scored);
			}

			set
			{
				scored = value;
			}
		}

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if(components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.btnRetrieve = new System.Windows.Forms.Button();
			this.lblRows = new System.Windows.Forms.Label();
			this.lblCols = new System.Windows.Forms.Label();
			this.lblTop = new System.Windows.Forms.Label();
			this.txtRows = new System.Windows.Forms.TextBox();
			this.txtCols = new System.Windows.Forms.TextBox();
			this.txtTop = new System.Windows.Forms.TextBox();
			this.dgScores = new System.Windows.Forms.DataGrid();
			this.btnOK = new System.Windows.Forms.Button();
			((System.ComponentModel.ISupportInitialize)(this.dgScores)).BeginInit();
			this.SuspendLayout();
			// 
			// btnRetrieve
			// 
			this.btnRetrieve.Location = new System.Drawing.Point(352, 104);
			this.btnRetrieve.Name = "btnRetrieve";
			this.btnRetrieve.Size = new System.Drawing.Size(128, 23);
			this.btnRetrieve.TabIndex = 1;
			this.btnRetrieve.Text = "Retrieve...";
			this.btnRetrieve.Click += new System.EventHandler(this.btnRetrieve_Click);
			// 
			// lblRows
			// 
			this.lblRows.AutoSize = true;
			this.lblRows.Location = new System.Drawing.Point(352, 8);
			this.lblRows.Name = "lblRows";
			this.lblRows.Size = new System.Drawing.Size(36, 13);
			this.lblRows.TabIndex = 2;
			this.lblRows.Text = "Rows:";
			// 
			// lblCols
			// 
			this.lblCols.AutoSize = true;
			this.lblCols.Location = new System.Drawing.Point(352, 40);
			this.lblCols.Name = "lblCols";
			this.lblCols.Size = new System.Drawing.Size(52, 13);
			this.lblCols.TabIndex = 3;
			this.lblCols.Text = "Columns:";
			// 
			// lblTop
			// 
			this.lblTop.AutoSize = true;
			this.lblTop.Location = new System.Drawing.Point(352, 72);
			this.lblTop.Name = "lblTop";
			this.lblTop.Size = new System.Drawing.Size(27, 13);
			this.lblTop.TabIndex = 4;
			this.lblTop.Text = "Top:";
			// 
			// txtRows
			// 
			this.txtRows.Location = new System.Drawing.Point(416, 8);
			this.txtRows.Name = "txtRows";
			this.txtRows.Size = new System.Drawing.Size(64, 20);
			this.txtRows.TabIndex = 5;
			this.txtRows.Text = "";
			this.txtRows.Validating += new System.ComponentModel.CancelEventHandler(this.txt_Validating);
			// 
			// txtCols
			// 
			this.txtCols.Location = new System.Drawing.Point(416, 40);
			this.txtCols.Name = "txtCols";
			this.txtCols.Size = new System.Drawing.Size(64, 20);
			this.txtCols.TabIndex = 6;
			this.txtCols.Text = "";
			this.txtCols.Validating += new System.ComponentModel.CancelEventHandler(this.txt_Validating);
			// 
			// txtTop
			// 
			this.txtTop.Location = new System.Drawing.Point(416, 72);
			this.txtTop.Name = "txtTop";
			this.txtTop.Size = new System.Drawing.Size(64, 20);
			this.txtTop.TabIndex = 7;
			this.txtTop.Text = "";
			this.txtTop.Validating += new System.ComponentModel.CancelEventHandler(this.txt_Validating);
			// 
			// dgScores
			// 
			this.dgScores.AllowSorting = false;
			this.dgScores.DataMember = "";
			this.dgScores.HeaderForeColor = System.Drawing.SystemColors.ControlText;
			this.dgScores.Location = new System.Drawing.Point(8, 8);
			this.dgScores.Name = "dgScores";
			this.dgScores.ReadOnly = true;
			this.dgScores.RowHeadersVisible = false;
			this.dgScores.Size = new System.Drawing.Size(336, 296);
			this.dgScores.TabIndex = 8;
			// 
			// btnOK
			// 
			this.btnOK.DialogResult = System.Windows.Forms.DialogResult.OK;
			this.btnOK.Location = new System.Drawing.Point(352, 272);
			this.btnOK.Name = "btnOK";
			this.btnOK.Size = new System.Drawing.Size(128, 32);
			this.btnOK.TabIndex = 9;
			this.btnOK.Text = "OK";
			// 
			// HighScores
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.ClientSize = new System.Drawing.Size(496, 309);
			this.Controls.AddRange(new System.Windows.Forms.Control[] {
											  this.btnOK,
											  this.dgScores,
											  this.txtTop,
											  this.txtCols,
											  this.txtRows,
											  this.lblTop,
											  this.lblCols,
											  this.lblRows,
											  this.btnRetrieve});
			this.Name = "HighScores";
			this.Text = "Scramble High Scores";
			this.Load += new System.EventHandler(this.HighScores_Load);
			((System.ComponentModel.ISupportInitialize)(this.dgScores)).EndInit();
			this.ResumeLayout(false);

		}
		#endregion

		/// <summary>
		/// Handle the Retrieve button: list the high scores for
		/// the specified image rows/columns.
		/// </summary>
		/// <param name="sender">Sender control (unused)</param>
		/// <param name="e">Event arguments (unused)</param>
		private void btnRetrieve_Click(object sender, System.EventArgs e)
		{
			SCScore[] scores;
			int i;
			DataSet ds;

			if (rows < 0 || cols < 0 || top < 0)
			{
				MessageBox.Show("Please enter valid values for rows, columns, and top!",
				    "Invalid rows, cols, top values",
				    MessageBoxButtons.OK,
				    MessageBoxIcon.Error);
				return;
			}

			high.Rows = rows;
			high.Columns = cols;
			high.ScoreNum = top;
			scores = high.SelectTop();
			ds = new DataSet();
			ds.Tables.Add("scores");
			ds.Tables[0].Columns.Add("Rank", typeof(int));
			ds.Tables[0].Columns.Add("Name", typeof(string));
			ds.Tables[0].Columns.Add("Moves", typeof(int));
			ds.Tables[0].Columns.Add("Date", typeof(DateTime));
			for (i = 0; i < scores.Length; i++)
				ds.Tables[0].Rows.Add(new object[] {
				    i + 1, scores[i].name,
				    scores[i].moves, scores[i].date});
			dgScores.DataSource = ds;
			dgScores.DataMember = "scores";
		}

		/// <summary>
		/// Add a high score entry using the web service.
		/// </summary>
		/// <param name="sc">The high score entry to add</param>
		/// <param name="conn">Database connection (unused)</param>
		/// <returns>The database ID of the newly-added entry.</returns>
		protected int SvcAddScore(ref SCScore sc, Connection conn)
		{
			sc.id = svc.AddScore(sc);
			return (sc.id);
		}

		/// <summary>
		/// Delete a high score entry using the web service.
		/// </summary>
		/// <param name="sc">The high score entry to delete</param>
		/// <param name="conn">Database connection (unused)</param>
		protected void SvcDeleteScore(SCScore sc, Connection conn)
		{
			svc.DeleteScore(sc);
		}

		/// <summary>
		/// List all high score entries using the web service.
		/// </summary>
		/// <param name="conn">Database connection (unused)</param>
		/// <returns>An array of all high score entries.</returns>
		protected SCScore[] SvcListScores(Connection conn)
		{
			return (svc.ListAllScores());
		}

		/// <summary>
		/// Validate a number entered into a textbox and store it
		/// into the appropriate member variable: rows, cols, top.
		/// </summary>
		/// <param name="sender">Sender control (txtRows, txtCols,
		/// or txtTop)</param>
		/// <param name="e">Event arguments (unused)</param>
		private void txt_Validating(object sender, System.ComponentModel.CancelEventArgs e)
		{
			TextBox tb;
			int v;

			tb = sender as TextBox;
			if (tb == null)
				return;
			try
			{
				v = int.Parse(tb.Text);
			}
			catch
			{
				e.Cancel = true;
				return;
			}
			if (v <= 0)
			{
				e.Cancel = true;
				return;
			}
			if (tb == txtRows)
				rows = v;
			else if (tb == txtCols)
				cols = v;
			else if (tb == txtTop)
				top = v;
		}

		/// <summary>
		/// Form load: if the scored flag is set, check if there is really
		/// a high score, and query the player name if so.
		/// </summary>
		/// <param name="sender">Sender control (unused)</param>
		/// <param name="e">Event arguments (unused)</param>
		private void HighScores_Load(object sender, System.EventArgs e)
		{
			HighScoreName dlg;
			SCScore s;

			if (!scored)
				return;
			scored = false;
			try
			{
				btnRetrieve_Click(sender, null);
				if (!high.IsHighScore(puzzle.Moves))
					return;
				dlg = new HighScoreName();
				dlg.Moves = puzzle.Moves;
				if (dlg.ShowDialog() != DialogResult.OK)
					return;
				s = new SCScore();
				s.name = dlg.PlayerName;
				s.picture = puzzle.Image.Image.Name;
				s.rows = rows;
				s.cols = cols;
				s.moves = puzzle.Moves;
				s.date = DateTime.Now;
				high.UpdateScore(s);
				btnRetrieve_Click(sender, null);
			}
			catch (Exception ex)
			{
				MessageBox.Show("Error updating the high score table: " +
				    ex.Message,
				    "High score update failed",
				    MessageBoxButtons.OK,
				    MessageBoxIcon.Error);
			}
		}
	}
}
