using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.IO;
using System.Windows.Forms;

using Ringlet.Drawing.Scramble;
using Ringlet.Misc.VCS;

namespace Ringlet.Misc.Scramble.Unscramble
{
	/// <summary>
	/// Summary description for PuzzleChoose.
	/// </summary>
	public class PuzzleChoose : System.Windows.Forms.Form, IVersioned
	{
		/// <summary>
		/// The list of the puzzles to choose from.
		/// </summary>
		private System.Windows.Forms.ListView lvPuzzles;
		/// <summary>
		/// The OK button.
		/// </summary>
		private System.Windows.Forms.Button btnOK;
		/// <summary>
		/// The Cancel button.
		/// </summary>
		private System.Windows.Forms.Button btnCancel;
		/// <summary>
		/// The Refresh button.
		/// </summary>
		private System.Windows.Forms.Button btnRefresh;
		/// <summary>
		/// Required designer variable.
		/// </summary>
		private System.ComponentModel.Container components = null;
		/// <summary>
		/// The web service to obtain the image data from.
		/// </summary>
		protected PuzzleData svc;
		/// <summary>
		/// The list of images.
		/// </summary>
		protected SCImageData[] images;
		/// <summary>
		/// The chosen image's data.
		/// </summary>
		protected SCImageData chosendata;

		/// <summary>
		/// Initialize a PuzzleChoose object with default values.
		/// </summary>
		public PuzzleChoose()
		{
			//
			// Required for Windows Form Designer support
			//
			InitializeComponent();

			svc = null;
			images = null;
			chosendata = new SCImageData();
			chosendata.id = -1;
		}

		/// <summary>
		/// Get the version control system tag.
		/// </summary>
		public virtual string VCSTag
		{
			get
			{
				return ("$Ringlet: cs/misc/Scramble/Unscramble/PuzzleChoose.cs,v 1.1 2003/07/11 22:16:38 roam Exp $");
			}
		}

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if(components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.lvPuzzles = new System.Windows.Forms.ListView();
			this.btnOK = new System.Windows.Forms.Button();
			this.btnCancel = new System.Windows.Forms.Button();
			this.btnRefresh = new System.Windows.Forms.Button();
			this.SuspendLayout();
			// 
			// lvPuzzles
			// 
			this.lvPuzzles.Location = new System.Drawing.Point(16, 24);
			this.lvPuzzles.Name = "lvPuzzles";
			this.lvPuzzles.Size = new System.Drawing.Size(424, 304);
			this.lvPuzzles.TabIndex = 0;
			// 
			// btnOK
			// 
			this.btnOK.DialogResult = System.Windows.Forms.DialogResult.OK;
			this.btnOK.Location = new System.Drawing.Point(472, 24);
			this.btnOK.Name = "btnOK";
			this.btnOK.Size = new System.Drawing.Size(88, 23);
			this.btnOK.TabIndex = 1;
			this.btnOK.Text = "OK";
			this.btnOK.Click += new System.EventHandler(this.btnOK_Click);
			// 
			// btnCancel
			// 
			this.btnCancel.DialogResult = System.Windows.Forms.DialogResult.Cancel;
			this.btnCancel.Location = new System.Drawing.Point(472, 64);
			this.btnCancel.Name = "btnCancel";
			this.btnCancel.Size = new System.Drawing.Size(88, 23);
			this.btnCancel.TabIndex = 2;
			this.btnCancel.Text = "Cancel";
			// 
			// btnRefresh
			// 
			this.btnRefresh.Location = new System.Drawing.Point(472, 104);
			this.btnRefresh.Name = "btnRefresh";
			this.btnRefresh.Size = new System.Drawing.Size(88, 23);
			this.btnRefresh.TabIndex = 3;
			this.btnRefresh.Text = "Refresh...";
			this.btnRefresh.Click += new System.EventHandler(this.btnRefresh_Click);
			// 
			// PuzzleChoose
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.ClientSize = new System.Drawing.Size(584, 421);
			this.Controls.AddRange(new System.Windows.Forms.Control[] {
											  this.btnRefresh,
											  this.btnCancel,
											  this.btnOK,
											  this.lvPuzzles});
			this.Name = "PuzzleChoose";
			this.Text = "Choose a puzzle";
			this.Load += new System.EventHandler(this.PuzzleChoose_Load);
			this.ResumeLayout(false);

		}
		#endregion

		/// <summary>
		/// List the puzzles on form load.
		/// </summary>
		/// <param name="sender">Sender control (passed to btnRefresh)</param>
		/// <param name="e">Event arguments (unused)</param>
		private void PuzzleChoose_Load(object sender, System.EventArgs e)
		{
			btnRefresh_Click(sender, null);
		}

		/// <summary>
		/// Query the web service for the image list and display
		/// the images along with text descriptions.
		/// </summary>
		/// <param name="sender">Sender control (unused, may be the form)</param>
		/// <param name="e">Event arguments (unused)</param>
		private void btnRefresh_Click(object sender, System.EventArgs e)
		{
			Image[] thumbs;
			bool update;
			int i;
			ImageList lst;
			byte[] buf;
			MemoryStream m;

			update = false;
			m = null;
			try
			{
				images = svc.ListImages();
				thumbs = new Image[images.Length];
				lst = new ImageList();
				for (i = 0; i < images.Length; i++)
				{
					buf = svc.GetImageThumbnail(
					    images[i].id);
					m = new MemoryStream(buf, false);
					thumbs[i] = new Bitmap(new Bitmap(m));
					m.Close();
					m = null;
					lst.Images.Add(thumbs[i]);
				}
				lvPuzzles.BeginUpdate();
				update = true;
				lvPuzzles.LargeImageList = lst;
				lvPuzzles.SmallImageList = null;
				lvPuzzles.Items.Clear();
				for (i = 0; i < images.Length; i++)
					lvPuzzles.Items.Add(
					    images[i].ToString(), i);
			}
			catch (Exception ex)
			{
				MessageBox.Show("Error: " + ex.Message,
				    "Error retrieving the image data",
				    MessageBoxButtons.OK,
				    MessageBoxIcon.Error);
			}
			finally
			{
				if (m != null)
					try
					{
						m.Close();
					}
					catch {}
				if (update)
				{
					lvPuzzles.EndUpdate();
					lvPuzzles.View = View.LargeIcon;
				}
			}
		}

		/// <summary>
		/// Handle the OK button: verify that exactly one puzzle has
		/// been chosen and close the form.
		/// </summary>
		/// <param name="sender">Sender control (unused)</param>
		/// <param name="e">Event arguments (unused)</param>
		private void btnOK_Click(object sender, System.EventArgs e)
		{
			if (lvPuzzles.SelectedItems.Count != 1)
			{
				MessageBox.Show("You must select a puzzle!",
				    "Select a puzzle to play",
				    MessageBoxButtons.OK,
				    MessageBoxIcon.Error);
				DialogResult = DialogResult.None;
			}
			chosendata = images[lvPuzzles.SelectedItems[0].Index];
			// Just in case the user made a wrong turn earlier...
			DialogResult = DialogResult.OK;
		}

		/// <summary>
		/// Get or set the webservice to use.
		/// </summary>
		public PuzzleData Service
		{
			get
			{
				return (svc);
			}

			set
			{
				svc = value;
			}
		}

		/// <summary>
		/// Get the chosen image ID.
		/// </summary>
		public int ImageID
		{
			get
			{
				return (chosendata.id);
			}
		}

		/// <summary>
		/// Get the chose image's data.
		/// </summary>
		public SCImageData ImageData
		{
			get
			{
				return (chosendata);
			}
		}
	}
}
