using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;

using Ringlet.Drawing.Scramble;
using Ringlet.Misc.VCS;

namespace Ringlet.Misc.Scramble.Unscramble
{
	/// <summary>
	/// Let the user specify the webservice properties.
	/// </summary>
	public class SvcProp : System.Windows.Forms.Form, IVersioned
	{
		/// <summary>
		/// Required designer variable.
		/// </summary>
		private System.ComponentModel.Container components = null;
		private System.Windows.Forms.Label lblURL;
		private System.Windows.Forms.TextBox txtURL;
		private System.Windows.Forms.Button btnOK;
		private System.Windows.Forms.Button btnCancel;
		private System.Windows.Forms.Button btnTest;
		/// <summary>
		/// The webservice properties.
		/// </summary>
		protected SvcProperties prop;

		/// <summary>
		/// Initialize a SvcProp object with default values.
		/// </summary>
		public SvcProp()
		{
			//
			// Required for Windows Form Designer support
			//
			InitializeComponent();

			Properties = new SvcProperties();
		}

		/// <summary>
		/// Initialize a SvcProp form with the specified properties.
		/// </summary>
		/// <param name="nprop">The webservice properties to use.</param>
		public SvcProp(SvcProperties nprop) : this()
		{
			Properties = nprop;
		}

		/// <summary>
		/// Get the version control system tag.
		/// </summary>
		public virtual string VCSTag
		{
			get
			{
				return ("$Ringlet: cs/misc/Scramble/Unscramble/SvcProp.cs,v 1.1 2003/07/11 22:16:38 roam Exp $");
			}
		}

		/// <summary>
		/// Get or set the webservice properties.
		/// </summary>
		public virtual SvcProperties Properties
		{
			get
			{
				return (prop);
			}

			set
			{
				prop = value;
				txtURL.Text = prop.URL;
			}
		}

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if(components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.lblURL = new System.Windows.Forms.Label();
			this.txtURL = new System.Windows.Forms.TextBox();
			this.btnOK = new System.Windows.Forms.Button();
			this.btnCancel = new System.Windows.Forms.Button();
			this.btnTest = new System.Windows.Forms.Button();
			this.SuspendLayout();
			// 
			// lblURL
			// 
			this.lblURL.AutoSize = true;
			this.lblURL.Location = new System.Drawing.Point(16, 48);
			this.lblURL.Name = "lblURL";
			this.lblURL.Size = new System.Drawing.Size(30, 13);
			this.lblURL.TabIndex = 0;
			this.lblURL.Text = "URL:";
			// 
			// txtURL
			// 
			this.txtURL.Location = new System.Drawing.Point(72, 48);
			this.txtURL.Name = "txtURL";
			this.txtURL.Size = new System.Drawing.Size(320, 20);
			this.txtURL.TabIndex = 1;
			this.txtURL.Text = "";
			// 
			// btnOK
			// 
			this.btnOK.DialogResult = System.Windows.Forms.DialogResult.OK;
			this.btnOK.Location = new System.Drawing.Point(448, 48);
			this.btnOK.Name = "btnOK";
			this.btnOK.TabIndex = 2;
			this.btnOK.Text = "OK";
			this.btnOK.Click += new System.EventHandler(this.btnOK_Click);
			// 
			// btnCancel
			// 
			this.btnCancel.DialogResult = System.Windows.Forms.DialogResult.Cancel;
			this.btnCancel.Location = new System.Drawing.Point(448, 88);
			this.btnCancel.Name = "btnCancel";
			this.btnCancel.TabIndex = 3;
			this.btnCancel.Text = "Cancel";
			// 
			// btnTest
			// 
			this.btnTest.Location = new System.Drawing.Point(448, 128);
			this.btnTest.Name = "btnTest";
			this.btnTest.TabIndex = 4;
			this.btnTest.Text = "Test...";
			this.btnTest.Click += new System.EventHandler(this.btnTest_Click);
			// 
			// SvcProp
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.ClientSize = new System.Drawing.Size(560, 273);
			this.Controls.AddRange(new System.Windows.Forms.Control[] {
											  this.btnTest,
											  this.btnCancel,
											  this.btnOK,
											  this.txtURL,
											  this.lblURL});
			this.Name = "SvcProp";
			this.Text = "Connect to a Scramble webserver";
			this.ResumeLayout(false);

		}
		#endregion

		private void btnTest_Click(object sender, System.EventArgs e)
		{
			PuzzleData d;
			string ver;

			prop.URL = txtURL.Text;
			try
			{
				prop.Create();
				d = prop.PuzzleData;
				ver = d.GetVCSTag();
				MessageBox.Show(
				    "Successful PuzzleData access:\n" + ver);
			}
			catch (Exception ex)
			{
				MessageBox.Show("An error occurred:\n" +
				    ex.Message,
				    "Web service access error",
				    MessageBoxButtons.OK,
				    MessageBoxIcon.Error);
			}
		}

		private void btnOK_Click(object sender, System.EventArgs e)
		{
			prop.URL = txtURL.Text;
		}
	}
}
