using System;
using System.Runtime.Remoting;
using System.Runtime.Remoting.Channels;
using System.Runtime.Remoting.Channels.Http;

using Ringlet.Misc.VCS;

namespace Ringlet.Misc.Scramble.Unscramble
{
	/// <summary>
	/// The webservice connection properties.
	/// </summary>
	public class SvcProperties : IVersioned
	{
		/// <summary>
		/// The URL to contact the webservice at.
		/// </summary>
		protected string url;
		/// <summary>
		/// The webservice object itself.
		/// </summary>
		protected PuzzleData data;

		/// <summary>
		/// Initialize a SvcProperties object with default values.
		/// </summary>
		public SvcProperties()
		{
			url = "http://localhost/scramble/Scramble/PuzzleData.asmx";
			data = null;
		}

		/// <summary>
		/// Get the version control system tag.
		/// </summary>
		public virtual string VCSTag
		{
			get
			{
				return ("$Ringlet: cs/misc/Scramble/Unscramble/SvcProperties.cs,v 1.1 2003/07/11 22:16:38 roam Exp $");
			}
		}

		/// <summary>
		/// Get or set the webservice URL.
		/// </summary>
		public virtual string URL
		{
			get
			{
				return (url);
			}

			set
			{
				url = value;
			}
		}

		/// <summary>
		/// Get the webservice puzzle data object.
		/// </summary>
		public virtual PuzzleData PuzzleData
		{
			get
			{
				if (data == null)
					Create();
				return (data);
			}
		}

		/// <summary>
		/// Create a webservice object.
		/// </summary>
		public virtual void Create()
		{
			data = new PuzzleData(url);
		}
	}
}
