using System;
using System.Collections;
using System.ComponentModel;
using System.Drawing;
using System.IO;
using System.Text.RegularExpressions;
using System.Windows.Forms;

using Ringlet.Drawing.Scramble;
using Ringlet.Misc.VCS;

namespace Ringlet.Misc.Scramble.Unscramble
{
	/// <summary>
	/// The Unscramble! application's main form.
	/// </summary>
	public class Unscramble : System.Windows.Forms.Form, IVersioned
	{
		private System.Windows.Forms.MainMenu mainMenu1;
		private System.Windows.Forms.MenuItem miPuzzle;
		private System.Windows.Forms.MenuItem miPuzzleExit;
		private System.Windows.Forms.MenuItem miHelp;
		private System.Windows.Forms.MenuItem miHelpAbout;
		/// <summary>
		/// Required designer variable.
		/// </summary>
		private System.ComponentModel.Container components = null;
		private System.Windows.Forms.MenuItem miPuzzleConn;
		/// <summary>
		/// The webservice properties.
		/// </summary>
		protected SvcProperties prop;
		private System.Windows.Forms.Button btnNew;
		private System.Windows.Forms.MenuItem miPuzzleNew;
		private System.Windows.Forms.Panel pnlPuzzle;
		/// <summary>
		/// The web service object to use.
		/// </summary>
		protected PuzzleData svc;
		/// <summary>
		/// Flag: is there a game in progress?
		/// </summary>
		protected bool inprogress;
		/// <summary>
		/// The data structure describing the current image.
		/// </summary>
		protected SCImageData imagedata;
		/// <summary>
		/// The currently displayed (and played) puzzle.
		/// </summary>
		protected SCPuzzle puzzle;
		/// <summary>
		/// The X coordinate of the New button relative tot he panel.
		/// </summary>
		protected int offBtnX;
		/// <summary>
		/// The Y coordinate of the New button relative to the panel.
		/// </summary>
		protected int offBtnY;
		/// <summary>
		/// The width of the form relative to the panel.
		/// </summary>
		protected int offFormX;
		/// <summary>
		/// The height of the form relative to the panel.
		/// </summary>
		protected int offFormY;
		/// <summary>
		/// The X coordinate of the Moves label relative to the panel.
		/// </summary>
		protected int offMovesX;
		/// <summary>
		/// The Y coordinate of the Moves label relative to the panel.
		/// </summary>
		protected int offMovesY;
		private System.Windows.Forms.Label lblMoves;
		private System.Windows.Forms.MenuItem miHigh;
		private System.Windows.Forms.Button btnHigh;
		/// <summary>
		/// The array of pictureboxen used to display the puzzle
		/// fragments.
		/// </summary>
		protected PictureBox[,] boxen;
		
		/// <summary>
		/// Initialize the Unscramble! form.
		/// </summary>
		public Unscramble()
		{
			//
			// Required for Windows Form Designer support
			//
			InitializeComponent();

			prop = new SvcProperties();
			svc = null;
			inprogress = false;
			imagedata = new SCImageData();
			imagedata.id = -1;
			puzzle = null;
			boxen = null;
			GatherResizeInfo();
			HaveService();
		}

		/// <summary>
		/// Get the version control system tag.
		/// </summary>
		public virtual string VCSTag
		{
			get
			{
				return ("$Ringlet: cs/misc/Scramble/Unscramble/Unscramble.cs,v 1.5 2003/07/12 01:55:39 roam Exp $");
			}
		}

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if (components != null) 
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.mainMenu1 = new System.Windows.Forms.MainMenu();
			this.miPuzzle = new System.Windows.Forms.MenuItem();
			this.miPuzzleConn = new System.Windows.Forms.MenuItem();
			this.miPuzzleNew = new System.Windows.Forms.MenuItem();
			this.miPuzzleExit = new System.Windows.Forms.MenuItem();
			this.miHelp = new System.Windows.Forms.MenuItem();
			this.miHelpAbout = new System.Windows.Forms.MenuItem();
			this.btnNew = new System.Windows.Forms.Button();
			this.pnlPuzzle = new System.Windows.Forms.Panel();
			this.lblMoves = new System.Windows.Forms.Label();
			this.miHigh = new System.Windows.Forms.MenuItem();
			this.btnHigh = new System.Windows.Forms.Button();
			this.SuspendLayout();
			// 
			// mainMenu1
			// 
			this.mainMenu1.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
												      this.miPuzzle,
												      this.miHigh,
												      this.miHelp});
			// 
			// miPuzzle
			// 
			this.miPuzzle.Index = 0;
			this.miPuzzle.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
												     this.miPuzzleConn,
												     this.miPuzzleNew,
												     this.miPuzzleExit});
			this.miPuzzle.Text = "Puzzle";
			// 
			// miPuzzleConn
			// 
			this.miPuzzleConn.Index = 0;
			this.miPuzzleConn.Text = "Connect...";
			this.miPuzzleConn.Click += new System.EventHandler(this.miPuzzleConn_Click);
			// 
			// miPuzzleNew
			// 
			this.miPuzzleNew.Index = 1;
			this.miPuzzleNew.Shortcut = System.Windows.Forms.Shortcut.CtrlN;
			this.miPuzzleNew.Text = "&New game";
			this.miPuzzleNew.Click += new System.EventHandler(this.miPuzzleNew_Click);
			// 
			// miPuzzleExit
			// 
			this.miPuzzleExit.Index = 2;
			this.miPuzzleExit.Shortcut = System.Windows.Forms.Shortcut.CtrlW;
			this.miPuzzleExit.Text = "E&xit";
			this.miPuzzleExit.Click += new System.EventHandler(this.miPuzzleExit_Click);
			// 
			// miHelp
			// 
			this.miHelp.Index = 2;
			this.miHelp.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
												   this.miHelpAbout});
			this.miHelp.Text = "Help";
			// 
			// miHelpAbout
			// 
			this.miHelpAbout.Index = 0;
			this.miHelpAbout.Text = "About Unscramble...";
			// 
			// btnNew
			// 
			this.btnNew.Location = new System.Drawing.Point(560, 8);
			this.btnNew.Name = "btnNew";
			this.btnNew.Size = new System.Drawing.Size(88, 23);
			this.btnNew.TabIndex = 0;
			this.btnNew.Text = "New game";
			this.btnNew.Click += new System.EventHandler(this.btnNew_Click);
			// 
			// pnlPuzzle
			// 
			this.pnlPuzzle.Location = new System.Drawing.Point(8, 8);
			this.pnlPuzzle.Name = "pnlPuzzle";
			this.pnlPuzzle.Size = new System.Drawing.Size(528, 296);
			this.pnlPuzzle.TabIndex = 1;
			// 
			// lblMoves
			// 
			this.lblMoves.AutoSize = true;
			this.lblMoves.Location = new System.Drawing.Point(560, 288);
			this.lblMoves.Name = "lblMoves";
			this.lblMoves.Size = new System.Drawing.Size(55, 13);
			this.lblMoves.TabIndex = 2;
			this.lblMoves.Text = "No moves";
			// 
			// miHigh
			// 
			this.miHigh.Index = 1;
			this.miHigh.Text = "High scores";
			this.miHigh.Click += new System.EventHandler(this.miHigh_Click);
			// 
			// btnHigh
			// 
			this.btnHigh.Location = new System.Drawing.Point(560, 48);
			this.btnHigh.Name = "btnHigh";
			this.btnHigh.Size = new System.Drawing.Size(88, 23);
			this.btnHigh.TabIndex = 3;
			this.btnHigh.Text = "High scores";
			this.btnHigh.Click += new System.EventHandler(this.btnHigh_Click);
			// 
			// Unscramble
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.ClientSize = new System.Drawing.Size(656, 313);
			this.Controls.AddRange(new System.Windows.Forms.Control[] {
											  this.btnHigh,
											  this.lblMoves,
											  this.pnlPuzzle,
											  this.btnNew});
			this.Menu = this.mainMenu1;
			this.Name = "Unscramble";
			this.Text = "Unscramble!";
			this.ResumeLayout(false);

		}
		#endregion

		/// <summary>
		/// The main entry point for the application.
		/// </summary>
		[STAThread]
		static void Main() 
		{
			Application.Run(new Unscramble());
		}

		private void miPuzzleConn_Click(object sender, System.EventArgs e)
		{
			SvcProp dlg;

			try
			{
				dlg = new SvcProp(prop);
				if (dlg.ShowDialog() != DialogResult.OK)
					return;
				prop = dlg.Properties;
				prop.Create();
				svc = prop.PuzzleData;
			}
			catch (Exception ex)
			{
				MessageBox.Show("An error occurred: " + ex.Message,
				    "Error creating the web service",
				    MessageBoxButtons.OK,
				    MessageBoxIcon.Error);
				svc = null;
			}
		}

		/// <summary>
		/// Handle the Puzzle >> Exit menu item.
		/// </summary>
		/// <param name="sender">Sender control (unused)</param>
		/// <param name="e">Event arguments (unused)</param>
		private void miPuzzleExit_Click(object sender, System.EventArgs e)
		{
			if (inprogress && !ConfirmAbort())
				return;
			Close();
		}

		/// <summary>
		/// Handle the Puzzle >> New game menu item.
		/// </summary>
		/// <param name="sender">Sender control (passed to btnNew)</param>
		/// <param name="e">Event arguments (unused)</param>
		private void miPuzzleNew_Click(object sender, System.EventArgs e)
		{
			btnNew_Click(sender, null);
		}

		/// <summary>
		/// Ask for confirmation when aborting a game.
		/// </summary>
		/// <returns>True if the user confirmed the game cancelling.</returns>
		protected virtual bool ConfirmAbort()
		{
			DialogResult res;

			res = MessageBox.Show(
			    "Are you sure you want to abort the current game?",
			    "Game in progress!",
			    MessageBoxButtons.YesNo, MessageBoxIcon.Warning);
			return (res == DialogResult.Yes);
		}

		/// <summary>
		/// Handle the New Game button.
		/// </summary>
		/// <param name="sender">Sender control (unused)</param>
		/// <param name="e">Event arguments (unused)</param>
		private void btnNew_Click(object sender, System.EventArgs e)
		{
			PuzzleChoose choose;

			if (inprogress)
			{
				if (!ConfirmAbort())
					return;
				// FIXME: btnCancel_Click() ?
				inprogress = false;
			}

			// Choose a puzzle and FIXME: load it
			if (!HaveService())
				return;
			try
			{
				choose = new PuzzleChoose();
				choose.Service = svc;
				if (choose.ShowDialog() != DialogResult.OK)
					return;
				imagedata = choose.ImageData;
				BuildPuzzle();
				puzzle.Scramble(
				    (int)puzzle.Image.Image.Rows *
				    (int)puzzle.Image.Image.Columns * 3 + 1);
				lblMoves.Text = "No moves";
				ClearBoxen();
				ResizeForPuzzle();
				DisplayPuzzle();
			}
			catch (Exception ex)
			{
				MessageBox.Show("Error: " + ex.Message,
				    "Error choosing a puzzle",
				    MessageBoxButtons.OK,
				    MessageBoxIcon.Error);
				return;
			}

			// Start a game
			inprogress = true;
		}

		/// <summary>
		/// Ensure that we have a valid webservice object.
		/// </summary>
		/// <returns>True if we have a webservice object.</returns>
		protected virtual bool HaveService()
		{
			SvcProp dlg;

			try
			{
				if (svc != null)
					return (true);
				dlg = new SvcProp(prop);
				if (dlg.ShowDialog() != DialogResult.OK)
					return (false);
				prop = dlg.Properties;
				prop.Create();
				svc = prop.PuzzleData;
			}
			catch (Exception ex)
			{
				MessageBox.Show("An error occurred: " + ex.Message,
				    "Error creating the web service",
				    MessageBoxButtons.OK,
				    MessageBoxIcon.Error);
				svc = null;
			}
			return (svc != null);
		}

		/// <summary>
		/// Build an SCPuzzle object out of the image data.
		/// </summary>
		protected virtual void BuildPuzzle()
		{
			Image[,] frags;
			SCPuzzle pz;
			byte[] buf;
			MemoryStream m;
			int x, y;

			m = null;
			try
			{
				frags = new Image[imagedata.rows, imagedata.cols];
				for (y = 0; y < imagedata.rows; y++)
					for (x = 0; x < imagedata.cols; x++)
					{
						buf = svc.GetImageFragment(
						    imagedata.id, x, y);
						m = new MemoryStream(buf, false);
						frags[y, x] = new Bitmap(new Bitmap(m));
						m.Close();
						m = null;
					}
				pz = new SCPuzzle();
				pz.DisableDB = true;
				pz.BuildPuzzle(imagedata, frags);
				puzzle = pz;
			}
			finally
			{
				if (m != null)
					try
					{
						m.Close();
					}
					catch {}
			}
		}

		/// <summary>
		/// Display the current puzzle.
		/// </summary>
		protected virtual void DisplayPuzzle()
		{
			int x, y;
			bool solved;

			if (boxen == null)
				CreateBoxen();

			solved = puzzle.IsOrdered();
			for (y = 0; y < imagedata.rows; y++)
				for (x = 0; x < imagedata.cols; x++)
				{
					boxen[y, x].Image =
					    puzzle.Image.Image.Pieces[
					    puzzle.Pieces[y, x].imy,
					    puzzle.Pieces[y, x].imx];
					boxen[y, x].Visible =
					    x != puzzle.X || y != puzzle.Y ||
					    solved;
				}
		}

		/// <summary>
		/// Clear the boxen array, removing all associations of
		/// its members with the outside world.
		/// </summary>
		protected virtual void ClearBoxen()
		{
			int x, y;

			if (boxen == null)
				return;
			for (y = 0; y <= boxen.GetUpperBound(0); y++)
				for (x = 0; x <= boxen.GetUpperBound(1); x++)
				{
					if (boxen[y, x] == null)
						continue;
					boxen[y, x].Visible = false;
					boxen[y, x].Image = null;
					pnlPuzzle.Controls.Remove(boxen[y, x]);
					boxen[y, x] = null;
				}
			boxen = null;
		}

		/// <summary>
		/// Create the picture boxes needed to display the puzzle
		/// in the appropriate places, with the appropriate sizes.
		/// </summary>
		protected virtual void CreateBoxen()
		{
			int x, y, left, top;
			PictureBox[,] baa;
			PictureBox b;
			
			baa = new PictureBox[imagedata.rows, imagedata.cols];
			top = pnlPuzzle.Top + 2;
			b = new PictureBox();
			for (y = 0; y < imagedata.rows; y++)
			{
				left = pnlPuzzle.Left + 2;
				for (x = 0; x < imagedata.cols; x++)
				{
					b = new PictureBox();
					b.Name = String.Format(
					    "pzpic_{0}_{1}", x, y);
					b.Visible = false;
					b.Height = puzzle.Image.Image.Pieces[y, x].Height;
					b.Width = puzzle.Image.Image.Pieces[y, x].Width;
					b.Image = puzzle.Image.Image.Pieces[y, x];
					b.Click += new EventHandler(BoxClick);
					b.Top = top;
					b.Left = left;
					pnlPuzzle.Controls.Add(b);
					b.Visible = true;	
					left += b.Image.Width + 2;
					baa[y, x] = b;
				}
				top += b.Image.Height + 2;
			}
			boxen = baa;
		}

		/// <summary>
		/// Handle a click on a puzzle picturebox and perform a move.
		/// </summary>
		/// <param name="sender">The sender object
		/// (hopefully a picturebox)</param>
		/// <param name="args">Event arguments (unused)</param>
		protected virtual void BoxClick(object sender, EventArgs args)
		{
			PictureBox b;
			int x, y, cx, cy;
			Regex re;
			Match m;
			SCDir dir;

			b = sender as PictureBox;
			if (b == null)
			{
				Control c = sender as Control;

				MessageBox.Show(
				    "Error: should have clicked a box, clicked a " +
				    sender.GetType().FullName + " named " +
				    (c == null? "(unknown)": c.Name) + " instead!",
				    "Click handler error",
				    MessageBoxButtons.OK,
				    MessageBoxIcon.Error);
				return;
			}
			re = new Regex("^pzpic_(\\d+)_(\\d+)$");
			m = re.Match(b.Name);
			if (m == null || m.Groups == null || m.Groups.Count != 3)
			{
				MessageBox.Show(
				    "Error: could not parse PictureBox name " +
				    b.Name,
				    "Could not parse PictureBox name",
				    MessageBoxButtons.OK,
				    MessageBoxIcon.Error);
				return;
			}
			x = int.Parse(m.Groups[1].Value);
			y = int.Parse(m.Groups[2].Value);
			cx = puzzle.X;
			cy = puzzle.Y;
			if (x == cx && y == cy - 1)
				dir = SCDir.Up;
			else if (x == cx && y == cy + 1)
				dir = SCDir.Down;
			else if (x == cx - 1 && y == cy)
				dir = SCDir.Left;
			else if (x == cx + 1 && y == cy)
				dir = SCDir.Right;
			else
			{
				MessageBox.Show("Invalid move...",
				    "Bad move, no cookie!",
				    MessageBoxButtons.OK,
				    MessageBoxIcon.Error);
				return;
			}

			// Perform the move
			puzzle.Move(dir);
			puzzle.Moves++;
			lblMoves.Text = String.Format("{0} move{1}",
			    puzzle.Moves, puzzle.Moves == 1? "": "s");

			// Shuffle the images
			b.Visible = false;
			b.Image = puzzle.Image.Image.Pieces[
			    puzzle.Pieces[y, x].imy,
			    puzzle.Pieces[y, x].imx];
			boxen[cy, cx].Image = puzzle.Image.Image.Pieces[
			    puzzle.Pieces[cy, cx].imy,
			    puzzle.Pieces[cy, cx].imx];
			boxen[cy, cx].Visible = true;

			if (puzzle.IsOrdered())
			{
				b.Visible = true;
				PuzzleSolved();
			}
		}

		/// <summary>
		/// Congratulate the player on a solved puzzle and
		/// display the highscores.
		/// </summary>
		protected virtual void PuzzleSolved()
		{
			HighScores dlg;

			inprogress = false;
			try
			{
				dlg = new HighScores();
				dlg.Service = svc;
				dlg.Puzzle = puzzle;
				dlg.Scored = true;
				dlg.ShowDialog();
			}
			catch (Exception e)
			{
				MessageBox.Show("Error: " + e.Message,
				    "High score display error",
				    MessageBoxButtons.OK,
				    MessageBoxIcon.Error);
			}
		}

		/// <summary>
		/// Analyze the components position relative to the panel
		/// so that we can keep them "in place" after resizing it
		/// to fit the puzzle.
		/// </summary>
		protected virtual void GatherResizeInfo()
		{
			offBtnX = btnNew.Left - pnlPuzzle.Right;
			offBtnY = btnNew.Top - pnlPuzzle.Top;
			offFormX = Width - pnlPuzzle.Right;
			offFormY = Height - pnlPuzzle.Bottom;
			offMovesX = lblMoves.Left - pnlPuzzle.Right;
			offMovesY = lblMoves.Top - pnlPuzzle.Bottom;
		}

		/// <summary>
		/// Resize the panel and rearrange the form's components
		/// to fit its new position and size.
		/// </summary>
		protected virtual void ResizeForPuzzle()
		{
			int w, h;
			int pnlWidth, pnlHeight, pnlRight, pnlBottom;
			int i;

			// Calculate w and h
			// FIXME: there ought to be a better way ;)
			for (i = 0, w = 0; i < imagedata.cols; i++)
				w += puzzle.Image.Image.Pieces[0, i].Width + 10;
			for (i = 0, h = 0; i < imagedata.rows; i++)
				h += puzzle.Image.Image.Pieces[i, 0].Height + 10;
			
			// Calculate the new panel coordinates
			// FIXME: 10?
			pnlWidth = w + 10;
			pnlHeight = h + 10;
			pnlRight = pnlPuzzle.Left + pnlWidth;
			pnlBottom = pnlPuzzle.Top + pnlHeight;

			// Resize the form...
			Width = pnlRight + offFormX;
			Height = pnlBottom + offFormY;
			btnNew.Left = pnlRight + offBtnX;
			btnNew.Top = pnlPuzzle.Top + offBtnY;
			btnHigh.Left = btnNew.Left;
			lblMoves.Left = pnlRight + offMovesX;
			lblMoves.Top = pnlBottom + offMovesY;

			// Resize the panel
			pnlPuzzle.Width = pnlWidth;
			pnlPuzzle.Height = pnlHeight;
			// See havoc and descruction ensue.
		}

		/// <summary>
		/// Handle the 'High scores' menu item: just invoke the btnHigh
		/// click handler.
		/// </summary>
		/// <param name="sender">Sender control (passed to btnHigh)</param>
		/// <param name="e">Event arguments (unused)</param>
		private void miHigh_Click(object sender, System.EventArgs e)
		{
			btnHigh_Click(sender, null);
		}

		/// <summary>
		/// Handle the High scores click.
		/// </summary>
		/// <param name="sender">Sender control (unused)</param>
		/// <param name="e">Event arguments (unused)</param>
		private void btnHigh_Click(object sender, System.EventArgs e)
		{
			HighScores dlg;

			dlg = new HighScores();
			dlg.Service = svc;
			if (puzzle != null)
				dlg.Puzzle = puzzle;
			dlg.ShowDialog();
		}
	}
}
