using System;
using System.Collections;
using System.Data;

using Ringlet.Data.ConnectionUtil;
using Ringlet.Misc.VCS;

namespace Ringlet.Drawing.Scramble
{
	/// <summary>
	/// Summary description for SCDatabase.
	/// </summary>
	public class SCDatabase : IVersioned
	{
		ConnectionProperties prop;
		Connection conn;

		/// <summary>
		/// Initialize an SCDatabase object with default values.
		/// </summary>
		public SCDatabase()
		{
			prop = null;
			conn = null;
		}

		/// <summary>
		/// Get the version control system tag.
		/// </summary>
		public virtual string VCSTag
		{
			get
			{
				return ("$Ringlet: cs/misc/Scramble/libscpic/SCDatabase.cs,v 1.1 2003/07/10 07:42:36 roam Exp $");
			}
		}

		/// <summary>
		/// Get or set the SQL connection properties.
		/// </summary>
		public ConnectionProperties ConnectionProperties
		{
			get
			{
				return (prop);
			}

			set
			{
				prop = value;
				if (conn != null)
					conn.Close();
				conn = new Connection(prop);
				conn.Create();
			}
		}

		/// <summary>
		/// Obtain a list of all images stored in the database.
		/// </summary>
		/// <returns>An array of image data structures.</returns>
		public SCImageData[] ListAllImages()
		{
			ConnectionState st;
			IDbCommand cmd;
			IDataReader rdr;
			ArrayList l;
			SCImageData[] list;
			SCImageData idata;

			st = ConnectionState.Closed;
			list = null;
			rdr = null;
			l = new ArrayList();
			try
			{
				cmd = conn.CreateCommand();
				cmd.CommandText = "SELECT * FROM scpics ORDER BY pic_id";
				st = conn.StateOpen();
				rdr = cmd.ExecuteReader();
				if (rdr == null)
				{
					conn.StateRestore(st);
					return (null);
				}
				while (rdr.Read())
				{
					idata = new SCImageData(rdr);
					l.Add(idata);
				}
				rdr.Close();
				rdr = null;

				list = (SCImageData[])l.ToArray(typeof(SCImageData));
			}
			finally
			{
				if (rdr != null)
					try
					{
						rdr.Close();
					}
					catch {}
				try
				{
					conn.StateRestore(st);
				}
				catch {}
			}
			return (list);
		}

		/// <summary>
		/// Obtain the data describing the image with the specified
		/// database ID.
		/// </summary>
		/// <param name="id">The image ID to query.</param>
		/// <returns>The image data.</returns>
		public SCImageData ListImage(int id)
		{
			SCImageData[] list;
			SCImageData nullimg;

			list = ListAllImages();
			foreach (SCImageData img in list)
				if (img.id == id)
					return (img);
			nullimg = new SCImageData();
			nullimg.id = -1;
			return (nullimg);
		}
	}
}
