using System;
using System.Data;

using Ringlet.Misc.VCS;

namespace Ringlet.Drawing.Scramble
{
	/// <summary>
	/// A data structure describing a fragmented image.
	/// </summary>
	public struct SCImageData : IVersioned
	{
		/// <summary>
		/// The database ID of the image.
		/// </summary>
		public int id;
		/// <summary>
		/// The human-readable name of the image.
		/// </summary>
		public string name;
		/// <summary>
		/// The human-readable comment of the image.
		/// </summary>
		public string comment;
		/// <summary>
		/// The name of the image file.
		/// </summary>
		public string filename;
		/// <summary>
		/// The directory where the image fragments are stored.
		/// </summary>
		public string filedir;
		/// <summary>
		/// The image fragment filename prefix.
		/// </summary>
		public string fileprefix;
		/// <summary>
		/// The image fragment filename suffix.
		/// </summary>
		public string filesuffix;
		/// <summary>
		/// The number of rows the image is fragmented into.
		/// </summary>
		public uint rows;
		/// <summary>
		/// The number of columns the image is fragmented into.
		/// </summary>
		public uint cols;

		/// <summary>
		/// Initialize an image data structure from a database record.
		/// </summary>
		/// <param name="rdr">The data reader to query.</param>
		public SCImageData(IDataReader rdr)
		{
			id = (int)rdr["pic_id"];
			name = (string)rdr["pic_name"];
			comment = (string)rdr["pic_comment"];
			filename = (string)rdr["pic_fname"];
			filedir = (string)rdr["pic_fragdir"];
			fileprefix = (string)rdr["pic_fragprefix"];
			filesuffix = (string)rdr["pic_fragsuffix"];
			cols = (uint)(int)rdr["pic_slx"];
			rows = (uint)(int)rdr["pic_sly"];
		}

		/// <summary>
		/// Get the version control system tag.
		/// </summary>
		public string VCSTag
		{
			get
			{
				return ("$Ringlet: cs/misc/Scramble/libscpic/SCImageData.cs,v 1.2 2003/07/11 22:11:10 roam Exp $");
			}
		}

		/// <summary>
		/// Provide a human-readable description of the image.
		/// </summary>
		/// <returns>A string containing the image name and size.</returns>
		public override string ToString()
		{
			return (String.Format("{0} ({1}x{2})",
			    name, cols, rows));
		}
	}
}
